# NATools - Package with parallel corpora tools
# Copyright (C) 2002-2004  Alberto Simes
#
# This package is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

package NAT::Translator::method2;

use strict;
use Data::Dumper;
use NAT::PCorpus;
use NAT::Dict;

our $margin = 3;

our $PT_EN;
our $EN_PT;

sub method {
  my ($self, $search_string, $source, $target) = @_;

  $PT_EN = $self->{sourceDB};
  $EN_PT = $self->{targetDB};

  my @search_words = split/\s+/, $search_string;

  $source =~ s/$search_string/join(" ",map {"_$_"} @search_words)/ei;
  return process($#search_words + 1, $source, $target);
}

sub words {
  return grep { $_ !~ m{[.,;!?]}} split /\s+/, shift;
}

sub process {
  my ($size, $source, $target) = @_;

  my @search_indexes = ();
  my @source = words($source);
  my $i = 0;
  @source = map {
    if (m!^_!) {
      push @search_indexes, $i;
      $_ = $';
    }
    $i++;
    "$_" } @source;

  my $low_search = $search_indexes[0];
  my $llow_search = $low_search - $margin;
  $llow_search = ($llow_search<0)?0:$llow_search;
  my $high_search = $search_indexes[-1];
  my $hhigh_search = $high_search + $margin;

  # my (%source,%swords);
  # @swords{@source} = @source;
  # @source{(0..$#source)} = @source;

  my @target = words($target);
  my (%target,%twords);
  $twords{$_} = 0 for @target;
  # @target{(0..$#target)} = @target;

  my @classifications;
  $i = 0;
  for my $word (@source) {
    my $trans = $PT_EN->translations($word);
    for (sort { $trans->{$b} <=> $trans->{$a} } keys %$trans) {
      if (exists($twords{$_})) {
	if ($i>=$llow_search && $i<=$hhigh_search) {
	  $twords{$_} = 0.8 * $trans->{$_};
	}
	### THIS CANNOT BE AN ELSIF
	if ($i>=$low_search && $i<=$high_search) {
	  $twords{$_} = $trans->{$_};
	}
	last;
      }
    }
    $i++;
  }

#  print Dumper(\%twords);
  return get_best_answer($size, \@target, \%twords);

#  print Dumper(\%twords);
#  print Dumper(\%search_indexes);
#  print Dumper(\%source);
#  print Dumper(\%target);

}

sub get_best_answer {
  my $winsize = shift;

  my @answer = @{ shift() };
  my %twords = %{ shift() };

  my %ans = ();

  for my $sz ($winsize-1..$winsize+1) {
    next unless $sz;
    my ($first_index, $score) = get_best_window($sz, \@answer, \%twords);
    $ans{create_window_str($sz, $first_index, \@answer)} = $score;
    # print $init,"\n";
    # print_window($sz,$init, @answer);
  }

  my $best_str = (sort {$ans{$b}<=>$ans{$a}} keys %ans)[0];
  my $best_val = $ans{$best_str};
  return ($best_str,$best_val);

}

sub get_best_window {
  my $size = shift;
  my @answer = @{shift()};
  my %twords = %{shift()};

  my %mean;
  for my $i (0..$#answer-$size+1) {
    $mean{$i} = NAT::mean0(@twords{@answer[$i..$i+$size-1]});
    # print "$i -- $mean{$i} -- "; print_window($size,$i,@answer);
  }
  my $first_index = (sort {$mean{$b} <=> $mean{$a}} keys %mean)[0];
  my $mean_value = $mean{$first_index};
  return ($first_index, $mean_value);
}

sub create_window_str {
  my ($window_size, $start_index, $sentence) = @_;
  join " ", @{$sentence}[$start_index..$start_index+$window_size-1];
}

sub print_window {
  my ($window_size, $start_index, @sentence) = @_;
  my $str = create_window_str($window_size, $start_index, @sentence);
  print "$str\n";
}

1;
