# -*- cperl -*-

use Test::More;
use Data::Dumper;
BEGIN { plan tests => 25 + 5*30 };

use_ok("NAT::PTD");

my $ptd = new NAT::PTD("t/02_ptd.dmp");
isa_ok($ptd,"NAT::PTD");
isa_ok($ptd,"NAT::PTD::Dumper");

# save it before messing with its contents
$ptd->saveAs(dmp    => "t/_out.dmp");
$ptd->saveAs(bz2    => "t/_out.dmp.bz2");
$ptd->saveAs(xz     => "t/_out.dmp.xz");
$ptd->saveAs(sqlite => "t/_out.sqlite");

# test original PTD
test_contents($ptd);

# test dumped PTD
ok(-f "t/_out.dmp");
my $optd = NAT::PTD->new("t/_out.dmp");
isa_ok($optd,"NAT::PTD");
isa_ok($optd,"NAT::PTD::Dumper");
test_contents($optd);
unlink "t/_out.dmp";

# test sqlite PTD
ok(-f "t/_out.sqlite");
my $sptd = NAT::PTD->new("t/_out.sqlite");
isa_ok($sptd,"NAT::PTD");
isa_ok($sptd,"NAT::PTD::SQLite");
test_contents($sptd);
unlink "t/_out.sqlite";


# test dumped and bzipped PTD
ok(-f "t/_out.dmp.bz2");
my $bzptd = NAT::PTD->new("t/_out.dmp.bz2");
isa_ok($bzptd,"NAT::PTD");
isa_ok($bzptd,"NAT::PTD::BzDmp");
test_contents($bzptd);
unlink "t/_out.dmp.bz2";


# test dumped and xz PTD
ok(-f "t/_out.dmp.xz");
my $xzptd = NAT::PTD->new("t/_out.dmp.xz");
isa_ok($xzptd,"NAT::PTD");
isa_ok($xzptd,"NAT::PTD::XzDmp");
test_contents($xzptd);
unlink "t/_out.dmp.xz";


## Reset things...
my $ptd1 = new NAT::PTD("t/02_ptd.dmp");
$ptd1->saveAs("sqlite", "t/02.sqlite");
my $ptd2 = new NAT::PTD("t/02.sqlite");

# Intersect with itself
$ptd2->intersect($ptd1);
is($ptd2->count("casa"), 100, "Intersect with itself maintains count of word");
is($ptd2->size, 250, "Intersect with itself maintains number of counts");

$ptd2->add($ptd1);
# adding duplicates occurrences
is($ptd2->count("casa"), 200);
is($ptd2->size, 500);
# adding with itself, maintains probabilities
is($ptd2->prob("casa","home"), .25);
is($ptd2->prob("casa","house"), .25);

$ptd1->subtractDomain("gato");
is_deeply([$ptd1->words],[qw.casa.]);

$ptd1->add($ptd2);
is($ptd1->size, 600);
is($ptd1->prob("casa","house"), .25);
is($ptd2->prob("casa","house"), .25);
unlink "t/02.sqlite";






sub test_contents {
    my $ptd = shift;

    ok(grep {$_ eq "casa"} $ptd->words, "casa is in the dictionary");
    ok(!(grep {$_ eq "albergue"} $ptd->words), "albergue not in the dictionary");

    is($ptd->count("casa"),     100, "Count casa = 100");
    is($ptd->count("albergue"), 0,   "Count albergue = 0");

    is($ptd->prob("casa","home"),  .25, "P(T(casa)=home)=.25");
    is($ptd->prob("casa","house"), .25, "P(T(casa)=house)=.25");
    is($ptd->prob("casa","homens"),  0, "P(T(casa)=homens)=.0");

    is_deeply([sort $ptd->trans("casa")], [qw.home house.], "T(casa)=[home,house]");
    is_deeply([sort $ptd->words],         [qw.casa gato.],  "Words=[casa,gato]");


    my $stats = $ptd->stats;
    is($stats->{size},  $ptd->size);
    is($stats->{occTotal}, $stats->{size});
    is($stats->{count}, $ptd->count);
    is($stats->{avgOcc}, 125);
    is($stats->{occMin}, 100);
    is($stats->{occMax}, 150);
    is($stats->{occMinWord}, 'casa');
    is($stats->{occMaxWord}, 'gato');
    is($stats->{probMax}, 1);
    is($stats->{probMin}, 0.25);
    is($stats->{avgBestTrans}, 1.25/2); # cross fingers

    ## Recalculate probabilities
    $ptd->reprob;
    is($ptd->count("casa"),100, "Count(casa) = 100 (R)"); # count maintains
    is($ptd->prob("casa","home"), .5);
    is($ptd->prob("casa","house"), .5);
    is($ptd->prob("casa","homens"), 0);

    is($ptd->size, 250, "DicSize = 250");

    $ptd->subtractDomain( ['gato'] );
    is_deeply([$ptd->words],['casa']);
    is($ptd->size, 100);
    is($ptd->count, 1);

    ##
    $ptd->downtr( sub {
                      my ($w,$c,%t) = @_;
                      $c = 0;
                      toentry($w,$c,%t);
                  }, filter => 1);

    is($ptd->count("casa"),0);

    $ptd->downtr( sub { return undef}, filter => 1);
    is_deeply([$ptd->words],[]);


}
